/*
 * This spell checker was inspired by work done by Garrison Locke, but
 * was rewritten almost completely by Chuck Hagenbuch to use
 * Prototype/Scriptaculous.
 *
 * $Horde: imp/js/src/SpellChecker.js,v 1.36.2.2 2008/01/02 11:31:15 jan Exp $
 *
 * Copyright 2005-2008 The Horde Project (http://www.horde.org/)
 *
 * See the enclosed file COPYING for license information (GPL). If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 */

var SpellChecker = Class.create();
SpellChecker.prototype = {
    // Vars used and defaulting to null:
    //   bad, choices, choicesDiv, curWord, htmlArea, htmlAreaParent, locale,
    //   localeChoices, onAfterSpellCheck, onBeforeSpellCheck, reviewDiv,
    //   statusButton, statusClass, suggestions, target, url
    options: {},
    resumeOnDblClick: true,
    state: 'CheckSpelling',

    initialize: function(url, target, statusButton, bs, locales, sc)
    {
        this.url = url;
        this.target = target;
        this.options.onComplete = this.onComplete.bind(this);
        this.statusButton = $(statusButton);
        this.statusButton.observe('click', this.process.bindAsEventListener(this));
        this.buttonStates = bs;

        if (locales) {
            var d = new Element('DIV').hide();
            d.id = 'spellcheckpopdown';
            d.addClassName('autocomplete');
            d.setStyle({ position: 'absolute' });

            var ul = new Element('UL');
            $H(locales).each(function(pair) {
                var li = new Element('LI', { lc: pair.key });
                li.update(pair.value);
                ul.appendChild(li);
            });
            d.appendChild(ul);

            this.localeChoices = new KeyNavList(d, { onChoose: this.setLocale.bindAsEventListener(this) });

            var popdown = new Element('A', { className: 'popdown' }).insert('&nbsp;');
            popdown.observe('click', function() {
                $('spellcheckpopdown').clonePosition(this.statusButton, {
                    setHeight: false,
                    setWidth: false,
                    offsetTop: this.statusButton.getHeight()
                });
                this.localeChoices.show();
            }.bind(this));

            this.statusButton.up().insertBefore(popdown, this.statusButton.next());

            document.body.appendChild(d);
        }

        this.statusClass = sc || '';

        // Make sure that the form gets turned back into real text
        // when it's submitted.
        $($(this.target).form).observe('submit', this.resume.bindAsEventListener(this));

        // We need to monitor clicks to know when to go out of
        // showSuggestions mode.
        document.observe('click', this.onClick.bindAsEventListener(this));

        this.status('CheckSpelling');
    },

    setLocale: function(e)
    {
        this.locale = e.getAttribute('lc');
    },

    targetValue: function(value)
    {
        // Ensure we have the correct input.
        var input = $(this.target);

        if (value) {
            input.value = value;
            return;
        }

        return (Object.isUndefined(input.value)) ? input.innerHTML : input.value;
    },

    process: function(e)
    {
        switch (this.state) {
        case 'CheckSpelling':
            this.spellCheck();
            break;

        case 'ResumeEdit':
            this.resume();
            break;
        }

        Event.stop(e);
    },

    spellCheck: function()
    {
        if (this.onBeforeSpellCheck) {
            this.onBeforeSpellCheck();
        }

        this.status('Checking');
        this.removeChoices();

        var p = $H();
        p.set(this.target, this.targetValue());
        this.options.parameters = p.toQueryString();

        var url = this.url;
        if (this.locale) {
            url += '/locale=' + this.locale;
        }
        if (this.htmlAreaParent) {
            url += '/html=1';
        }

        new Ajax.Request(url, this.options);
    },

    onClick: function(e)
    {
        // If the language dropdown is currently active, and the click
        // got here, then we should hide the language dropdown.
        if (this.localeChoices) {
            this.localeChoices.hide();
        }

        // If we're not currently showing any choices, then we return
        // and let the click go.
        if (!this.choicesDiv) {
            return true;
        }

        // If the click was on the word that we're currently showing
        // results for, then we do nothing and let the list be
        // redisplayed by the link's own onclick handler.
        var link = Event.findElement(e, 'SPAN');
        if (link && link == this.curWord) {
            return true;
        }

        // The KeyNavList will have already handled any valid clicks,
        // so that just leaves the rest of the window - in this case
        // we want to hide the KeyNavList and reset curWord without
        // changing anything.
        this.removeChoices();
        this.curWord = null;
    },

    falsifyLinks: function()
    {
        $A(this.reviewDiv.getElementsByTagName('A')).each(function(e) {
            $(e).observe('click', Event.stop);
        });
    },

    unfalsifyLinks: function()
    {
        $A(this.reviewDiv.getElementsByTagName('A')).each(function(e) {
            $(e).stopObserving('click', Event.stop);
        });
    },

    onComplete: function(request)
    {
        var content,
            i = 0,
            input = $(this.target),
            reEscape = new RegExp('([().+*^$])'),
            result = request.responseText.evalJSON(true);

        this.removeChoices();

        if (Object.isUndefined(result)) {
            this.resume();
            this.status('Error');
            return;
        }

        var bad = result.bad || [];
        this.suggestions = result.suggestions || [];

        content = this.targetValue();
        if (this.htmlAreaParent) {
            content = content.replace(/\r?\n/g, '');
        } else {
            content = content.replace(/\r?\n/g, '~~~').escapeHTML();
        }

        $A(bad).each(function(node) {
            var re = new RegExp("(?:^|\\b)" + node.replace(reEscape, '\\' + RegExp.$1) + "(?:\\b|$)", 'g');
            content = content.replace(re, '<span index="' + i + '" id="' + this.target + (i++) + '" name="incorrect" class="incorrect">' + node + '</span>');
            i++;
        }.bind(this));

        if (!this.reviewDiv) {
            this.reviewDiv = new Element('div');
            this.reviewDiv.addClassName(input.className);
            this.reviewDiv.addClassName('spellcheck');
            this.reviewDiv.setStyle({ overflow: 'auto' });
            if (this.resumeOnDblClick) {
                this.reviewDiv.observe('dblclick', this.resume.bind(this));
            }
        }

        var d = input.getDimensions();
        this.reviewDiv.setStyle({ width: d.width + 'px', height: d.height + 'px'});

        if (!this.htmlAreaParent) {
            content = content.replace(/~~~/g, '<br />');
        }
        this.reviewDiv.update(content);

        // Now attach results behavior to each link.
        this.reviewDiv.select('span[name="incorrect"]').each(function(e) {
            e.observe('click', this.showSuggestions.bindAsEventListener(this));
        }.bind(this));

        this.falsifyLinks();

        if (this.htmlAreaParent) {
            this.htmlArea.hide();
            $(this.htmlAreaParent).appendChild(this.reviewDiv);
        } else {
            input.hide();
            input.parentNode.insertBefore(this.reviewDiv, input);
        }

        this.status('ResumeEdit');
    },

    showSuggestions: function(e)
    {
        this.curWord = Event.findElement(e, 'SPAN');

        this.removeChoices();

        this.choicesDiv = $(document.createElement('DIV')).hide();
        this.choicesDiv.addClassName('autocomplete');

        var pos = Position.page(this.curWord);
        this.choicesDiv.setStyle({ left: pos[0] + 'px', top: (pos[1] + 16) + 'px' });

        var ul = document.createElement('UL');
        $A(this.suggestions[this.curWord.getAttribute('index')]).each(function(node) {
            var li = document.createElement('LI');
            li.appendChild(document.createTextNode(node));
            ul.appendChild(li);
        });

        this.choicesDiv.appendChild(ul);
        this.choices = new KeyNavList(this.choicesDiv, { onChoose: function(event) { this.replaceWord(event, this.curWord); }.bind(this) });
        this.choices.show();
        document.body.appendChild(this.choicesDiv);
    },

    replaceWord: function(li, word)
    {
        word.update(li.innerHTML);
        word.className = 'corrected';
        this.removeChoices();
    },

    removeChoices: function()
    {
        if (this.choicesDiv) {
            this.choicesDiv.remove()
            this.choicesDiv = null;
        }
    },

    resume: function()
    {
        this.removeChoices();

        if (!this.reviewDiv) {
            return;
        }

        var input = $(this.target);

        this.reviewDiv.select('span[name="incorrect"]').each(function(n) {
            n.up().replaceChild(document.createTextNode(n.innerHTML), n);
        });

        this.unfalsifyLinks();

        var t = this.reviewDiv.innerHTML;
        if (!this.htmlAreaParent) {
            t = t.replace(/<br *\/?>/gi, '~~~').unescapeHTML().replace(/~~~/g, "\n");
        }
        this.targetValue(t);
        input.disabled = false;

        this.reviewDiv.remove();
        this.reviewDiv = null;

        this.status('CheckSpelling');

        if (this.htmlAreaParent) {
            this.htmlArea.show();
        } else {
            input.show();
        }

        if (this.onAfterSpellCheck) {
            this.onAfterSpellCheck();
        }
    },

    status: function(state)
    {
        if (!this.statusButton) {
            return;
        }

        this.state = state;
        switch (this.statusButton.tagName) {
        case 'INPUT':
            this.statusButton.value = this.buttonStates[state];
            break;

        case 'A':
            this.statusButton.update(this.buttonStates[state]);
            break;
        }
        this.statusButton.className = this.statusClass + ' ' + state;
    }

};
