/**
 * Provides the javascript for the compose.php script.
 *
 * $Horde: imp/js/src/compose.js,v 1.15.2.1 2007/12/20 13:59:22 jan Exp $
 *
 * See the enclosed file COPYING for license information (GPL). If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 */

/*
 * Variables defined in compose.php:
 *    cancel_url, compose_spellcheck, cursor_pos, identities, max_attachments,
 *    popup, redirect, rtemode, smf_check
 */

var display_unload_warning = true,
    spellcheck_done = false;

/**
 * Retrieve the editor object.
 */
function _getHtmlEditor()
{
    var ed = Try.these(
        function() { return __editors.message; },
        function() { return _editors.message; },
        function() { return _editors[$('message')]; }
    );

    if (!ed) {
        throw 'Unable to find the HTML Editor object, cannot continue.';
    }

    return ed;
}

function confirmCancel(event)
{
    if (window.confirm(ImpText.compose_cancel)) {
        display_unload_warning = false;
        if (popup) {
            if (cancel_url) {
                self.location.href = cancel_url;
            } else {
                self.close();
            }
        } else {
            window.location = cancel_url;
        }
        return true;
    } else {
        event.stop();
        return false;
    }
}

/**
 * Sets the cursor to the given position.
 */
function setCursorPosition(input, position)
{
    if (input.setSelectionRange) {
        /* This works in Mozilla */
        Field.focus(input);
        input.setSelectionRange(position, position);
    } else if (input.createTextRange) {
        /* This works in IE */
        var range = input.createTextRange();
        range.collapse(true);
        range.moveStart('character', position);
        range.moveEnd('character', 0);
        Field.select(range);
        range.scrollIntoView(true);
    }
}

function redirectSubmit(event)
{
    if ($F('to') == '') {
        alert(ImpText.compose_recipient);
        $('to').focus();
        event.stop();
        return false;
    }

    $('redirect').setStyle({ cursor: 'wait' });
    display_unload_warning = false;

    return true;
}

function change_identity(id)
{
    var last = identities[$F('last_identity')],
        next = identities[id],
        msg, ed, lastSignature, nextSignature;

    // If the rich text editor is on, we'll use a regexp to find the
    // signature comment and replace its contents.
    if (rtemode) {
        ed = _getHtmlEditor();
        msg = ed.getHTML().replace(/\r\n/g, '\n');

        lastSignature = '<p><!--begin_signature--><!--end_signature--></p>';
        nextSignature = '<p><!--begin_signature-->' + next[0].replace(/^ ?<br \/>\n/, '').replace(/ +/g, ' ') + '<!--end_signature--></p>';

        // Dot-all functionality achieved with [\s\S], see:
        // http://simonwillison.net/2004/Sep/20/newlines/
        msg = msg.replace(/<p>\s*<!--begin_signature-->[\s\S]*?<!--end_signature-->\s*<\/p>/, lastSignature);
    } else {
        msg = $F('message').replace(/\r\n/g, '\n');

        lastSignature = last[0].replace(/^\n/, '');
        nextSignature = next[0].replace(/^\n/, '');
    }

    var pos = (last[1]) ? msg.indexOf(lastSignature) : msg.lastIndexOf(lastSignature);
    if (pos != -1) {
        if (next[1] == last[1]) {
            msg = msg.substring(0, pos) + nextSignature + msg.substring(pos + lastSignature.length, msg.length);
        } else if (next[1]) {
            msg = nextSignature + msg.substring(0, pos) + msg.substring(pos + lastSignature.length, msg.length);
        } else {
            msg = msg.substring(0, pos) + msg.substring(pos + lastSignature.length, msg.length) + nextSignature;
        }

        msg = msg.replace(/\r\n/g, '\n').replace(/\n/g, '\r\n');

        $('last_identity').setValue(id);
        window.status = ImpText.compose_sigreplace;
    } else {
        window.status = ImpText.compose_signotreplace;
    }

    if (rtemode) {
        ed.setHTML(msg);
    } else {
        $('message').setValue(msg);
    }

    var smf = $('sent_mail_folder');
    if (smf_check) {
        var i = 0;
        $A(smf.options).detect(function(f) {
            if (f.value == next[2]) {
                smf.selectedIndex = i;
                return true;
            }
            ++i;
        });
    } else {
        if (smf.firstChild) {
            smf.replaceChild(document.createTextNode(next[2]), smf.firstChild);
        } else {
            smf.appendChild(document.createTextNode(next[2]));
        }
    }

    var save = $('ssm');
    if (save) {
        save.checked = next[3];
    }
    var bcc = $('bcc');
    if (bcc) {
        bccval = bcc.value;

        if (last[4]) {
            var re = new RegExp(last[4] + ",? ?", 'gi');
            bccval = bccval.replace(re, "");
            if (bccval) {
                bccval = bccval.replace(/, ?$/, "");
            }
        }

        if (next[4]) {
            if (bccval) {
                bccval += ', ';
            }
            bccval += next[4];
        }

        bcc.setValue(bccval);
    }
}

function uniqSubmit(actionID, event)
{
    if (actionID == 'send_message') {
        if (($F('subject') == '') &&
            !window.confirm(ImpText.compose_nosubject)) {
            if (event) {
                event.stop();
            }
            return false;
        }

        if (compose_spellcheck && !spellcheck_done) {
            /* TODO: Automatically send when finished with spellcheck. */
            SpellCheckerObject.spellCheck();
            spellcheck_done = true;
            if (event) {
                event.stop();
            }
            return false;
        }
    }

    $('compose').setStyle({ cursor: 'wait' });
    display_unload_warning = false;
    $('actionID').setValue(actionID);
    $('compose').submit();
}

function attachmentChanged()
{
    var fields = [], usedFields = 0;

    $A(document.compose.elements).each(function(i) {
        if (i.type == 'file' && i.name.startsWith('upload_')) {
            fields[fields.length] = i;
        }
    });

    if (max_attachments !== null &&
        fields.length == max_attachments) {
        return;
    }

    fields.each(function(i) {
        if (i.value.length > 0) {
            usedFields++;
        }
    });

    if (usedFields == fields.length) {
        var lastRow = $('attachment_row_' + usedFields);
        if (lastRow) {
            var td = new Element('TD', { align: 'left' }).insert(new Element('STRONG').insert(ImpText.compose_file + ' ' + (usedFields + 1) + ':')).insert('&nbsp;')

            var file = new Element('INPUT', { type: 'file', name: 'upload_' + (usedFields + 1), size: 25 });
            file.observe('change', attachmentChanged);
            td.insert(file);

            var select = new Element('SELECT', { name: 'upload_disposition_' + (usedFields + 1) });
            select.options[0] = new Option(ImpText.compose_attachment, 'attachment', true);
            select.options[1] = new Option(ImpText.compose_inline, 'inline');

            var newRow = new Element('TR', { id: 'attachment_row_' + (usedFields + 1) }).insert(td).insert(new Element('TD', { align: 'left' }).insert(select));

            lastRow.parentNode.insertBefore(newRow, lastRow.nextSibling);
        }
    }
}

function initializeSpellChecker()
{
    if (typeof SpellCheckerObject != 'object') {
        // If we fired before the onload that initializes the
        // spellcheck, wait.
        setTimeout(initializeSpellChecker, 100);
        return;
    }

    SpellCheckerObject.onBeforeSpellCheck = function() {
        // TODO: BC break as xinha has changed its internal storage structure
        // to the former.
        var ed = _getHtmlEditor();
        SpellCheckerObject.htmlArea = ed._htmlArea;
        SpellCheckerObject.htmlAreaParent = 'messageParent';
        ed._textArea.value = ed.outwardHtml(ed.getHTML());
    }
    SpellCheckerObject.onAfterSpellCheck = function() {
        var ed = _getHtmlEditor();
        SpellCheckerObject.htmlArea = SpellCheckerObject.htmlAreaParent = null;
        ed.setHTML(ed.inwardHtml(ed._textArea.value));
    }
}

/**
 * Code to run on window load.
 */
document.observe('dom:loaded', function() {
    /* Prevent Return from sending messages - it should bring us out of
     * autocomplete, not submit the whole form. */
    $$('INPUT').each(function(i) {
        /* Attach to everything but button and submit elements. */
        if (i.type != 'submit' && i.type != 'button') {
            i.observe('keydown', function(e) {
                if (e.keyCode == 10 || e.keyCode == Event.KEY_RETURN) {
                    e.stop();
                    return false;
                }
            });
        }
    });

    if (cursor_pos !== null) {
        setCursorPosition($('message'), cursor_pos);
    }

    if (redirect) {
        $('to').focus();
    } else {
        if (Prototype.Browser.IE) {
            $('subject').observe('keydown', function(e) {
                if (e.keyCode == Event.KEY_TAB && !e.shiftKey) {
                    e.stop();
                    $('message').focus();
                }
            });
        }

        if (rtemode) {
            initializeSpellChecker();
        }

        if ($('to') && !$F('to')) {
            $('to').focus();
        } else if (!$F('subject')) {
            if (rtemode) {
                $('subject').focus();
            } else {
                $('message').focus();
            }
        }
    }
});

/**
 * Warn before closing the window.
 */
Event.observe(window, 'beforeunload', function() {
    if (display_unload_warning) {
        return ImpText.compose_discard;
    }
});
